##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""Basic payment tool.
$Id: CreditTool.py,v 1.1.1.1 2002/02/12 12:09:56 thomas Exp $
"""
__version__='$Revision: 1.1.1.1 $'[11:-2]

import zLOG
from Products.CMFCore.utils import UniqueObject, getToolByName, _getAuthenticatedUser
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from ZODB.PersistentMapping import PersistentMapping
from Globals import package_home, Acquisition, Persistent, DTMLFile
import Globals #, os
from AccessControl.Role import RoleManager
from BTree import BTree
from ZPublisher.Converters import type_converters
from Acquisition import aq_inner, aq_parent, aq_base
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import ViewManagementScreens, setDefaultRoles
from Products.CMFCore import CMFCorePermissions
from General import _dtmldir

_marker = []  # Create a new marker object.

#_dtmldir = os.path.join( package_home( globals() ), 'dtml' )

ManagePayments = 'Register and clear payments'
setDefaultRoles(ManagePayments, ('Manager'))

ManageTransactions = 'Clear and register usage'
setDefaultRoles(ManageTransactions, ('Manager'))

class PaymentTool (UniqueObject, Folder):
    '''This tool wraps user objects, making them act as Member objects.
    '''
    id = 'portal_payment'
    meta_type = 'Simple Payment Tool'
    _v_temps = None
    _properties = ()

    # 
    # Percent that is given to the artists
    #
    split=75

    _guids = None

    NULL_GUID = "00000000-0000-0000-0000-000000000000"

    security = ClassSecurityInfo()

    _defaultMap = {'registerUserPayment': 'register_payment',
	           'listPayments'       : 'db_list_payments',
	           'clearPayment'       : 'clear_payment',
	           'retractPayment'     : 'retract_payment',
	           'registerTransaction': 'register_transaction',
	           'clearUserDept'      : 'clear_dept',
	           'viewCredits'        : 'view_credit',
		   'viewMoney'		: 'view_money',
	           'findResource'       : 'find_resource',
		   'getUserDept'        : 'get_dept',
		   'payDept'            : 'pay_dept'
	           }
    _map = None # PersistentMapping

    #
    # Tabs in the management screen.
    #
    manage_options=( ( { 'label' : 'Overview'
                       , 'action' : 'manage_overview'
                       }
                     , { 'label' : 'Payment Splitting'
                       , 'action' : 'manage_split'
                       }
		     , { 'label' : 'Dept'
		       , 'action' : 'manage_showDept'
		       }
		     , { 'label' : 'Payment Handling'
		       , 'action' : 'manage_handle'
		       }
		     , { 'label' : 'Test'
		       , 'action' : 'manage_test'
		       }
                     )
                   + Folder.manage_options
                   )

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = DTMLFile( 'explainPaymentTool', _dtmldir )


    security.declareProtected( ManagePayments
                             , 'manage_pending')
    manage_pending = DTMLFile('paymentsPending', _dtmldir )


    security.declareProtected( ManagePayments
                             , 'manage_showDept')
    manage_showDept = DTMLFile('paymentDept', _dtmldir )


    _manage_handle = DTMLFile('paymentHandling', _dtmldir )

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_handle')

    def manage_handle(self, REQUEST, manage_tabs_message=None):
       """
	 This is the view form for payment handling.
	 Payment handling is the map that desides what method goes where.
       """
       if self._map is None:
	 mp = PersistentMapping()
       else:
	 mp = self._map
       map = []
       for item in self._defaultMap.keys():
         map.append({'key': item,
	             'handler': mp.get(item, self._defaultMap[item])
		    })
       zLOG.LOG("CreditTool", 0, "teretrr")
       return self._manage_handle(REQUEST,
                                   map=map,
				   management_view = 'Payment Handling',
				   manage_tabs_message=manage_tabs_message)


    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_changeHandle' )
    def manage_changeHandle(self, props=None, REQUEST=None):
       """
	 This is changes the payment handling.
	 If a property is not set or set to "(Default)" the defaultMap
	 is used.
       """
       if props is None:
	 props = REQUEST
       map = PersistentMapping(self._defaultMap)
       for key in props.keys():
	 prop = strip(props.get(key, "(Default)"))
	 if not (prop == "" or prop == "(Default)"):
	   map[key] = prop

       self._map = map
       if REQUEST is not None:
         return self.manage_handle(REQUEST, manage_tabs_message="Changed")


    #
    # A simple test dtml. This is only for debuging purposes
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_test')
    manage_test = DTMLFile('paymentTest', _dtmldir )

    #
    # Management screen to change the percent of credits given to 
    # artists.
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_split')
    manage_split = DTMLFile('paymentSplit', _dtmldir )

    
    security.declareProtected(ManagePayments, 'registerSplit')
    def registerSplit(self, split, REQUEST=None, RESPONSE=None):
        '''
	  Changes the percent that artists get.
	'''
	self.split = int(split)

        if REQUEST is not None:
	  if RESPONSE is not None:
	    RESPONSE.redirect(REQUEST['HTTP_REFERER'])
	  else:
            return "Cleared"

    security.declarePublic('getSplit')
    def getSplit(self):
	"""
	  Returns the percent of the credits that should be given to
	  the artists.
	"""
	return self.split


    security.declareProtected( CMFCorePermissions.ViewManagementScreens
                             , 'getContentsInformation',)


    def __init__(self):
        self._members = BTree()
	self._supliers = BTree()

    def _getProp(self, key, default=None):
	if self._map is None:
	  return self._defaultMap.get(key, default)

        return self._map.get(key, self._defaultMap.get(key, default))


    security.declarePublic('setGUID')
    def setGUID(self, guid):
	"""
	  Maintains the GUID of the user.
	  The RelServer PPV plugin uses this to find the user
	  requesting a media file.
	  NOTE: This functionality should be in the Membership Data Tool.
	"""
	if self._guids is None:
          self._guids = PersistentMapping();
	if not (guid == self.NULL_GUID):
	  self._guids[guid] = _getAuthenticatedUser(self).getUserName();

    def getUser(self, guid):
	"""
	  Returns a username given a GUID.
	  Currently this function is only used by the RealServer PPV plugin.
	  NOTE: This functionality should be in the Membership Data Tool.
	"""
        if self._guids is None:
	  return "Guest"
        
        return self._guids.get(guid, default="Guest") 

    #
    # Payment registering and clearing
    #
    security.declarePublic('clearBadCredits')
    def clearBadCredits(self):
	"""
	  This method is meant to be called if a user has used to many credits.
	  When this method is called any such credits are marked as payed.
	  Proxy method who's implementation is dependant of the Payment
	  Handling.
	"""
	user = _getAuthenticatedUser(self).getId()
	funcName = self._getProp('clearBadCredits')
	func = getattr(self, funcName,None)
	zLOG.LOG("PaymentTool", 0, "REGISTER %s" % funcName)
	if func is not None:
	  func(user)
	
    security.declarePublic('getMoney')
    def getMoney(self):
	"""
	  Returns the unused credits converted to money for the current
	  user.
	"""
	return self.getUserMoney(_getAuthenticatedUser(self).getId())

    security.declareProtected(ManagePayments, 'getUserMoney')
    def getUserMoney(self, user):
	"""
	  Returns the unused credits converted to money for a given user.
	  Proxy method who's implementation is dependant of the Payment
	  Handling.
	"""
	funcName = self._getProp('viewMoney')
	func = getattr(self, funcName,None)
	zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
	if func is not None:
	  return func(user)
        
        return 0

    security.declarePublic('getCredits')
    def getCredits(self):
	"""
	  Returns the unused credits for the current user.
	"""
	return self.getUserCredits(_getAuthenticatedUser(self).getId())


    security.declareProtected(ManagePayments, 'getUserCredits')
    def getUserCredits(self, user):
	"""
	  Returns the unused credits for a given user.
	  Proxy method who's implementation is dependant of the Payment
	  Handling.
	"""
	funcName = self._getProp('viewCredits')
	func = getattr(self, funcName,None)
	zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
	if func is not None:
	  return func(user)
        
        return 0
	
    security.declareProtected(ManagePayments, 'registerUserPayment')
    def registerUserPayment(self, user, credit, rate, REQUEST=None, RESPONSE=None):
        '''
	  Registers a payment for a given user in the system
	  Proxy method who's implementation is dependant of the Payment
	  Handling.
	'''
	credit = int(credit)
	funcName = self._getProp('registerUserPayment')
	func = getattr(self, funcName,None)
	zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
	if func is not None:
	  zLOG.LOG("PaymentTool", 0, "Before call %s %d %s" % (user,credit, rate))
	  func(user,credit,rate)
	  zLOG.LOG("PaymentTool", 0, "After call")

        if REQUEST is not None:
	  if RESPONSE is not None:
	    RESPONSE.redirect(REQUEST['HTTP_REFERER'])
	  else:
            return "Cleared"

    security.declareProtected(ManagePayments, 'listPayments')
    def listPayments(self):
        '''
	  List the payments currently in the system.
	  Proxy method who's implementation is dependant of the Payment
	  Handling.
	'''
	funcName = self._getProp('listPayments')
	func = getattr(self, funcName,None)
	zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
	if func is not None:
	  return func()

        return ()
	
    security.declareProtected(ManagePayments, 'clearPayment')
    def clearPayment(self, id, REQUEST=None, RESPONSE=None):
       """
	 Clears a payment for further handling down the line.
	 A payment can have 3 states waiting, retracted and cleared.
	 Proxy method who's implementation is dependant of the Payment
	 Handling.
       """
       funcName = self._getProp('clearPayment')
       func = getattr(self, funcName,None)
       zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
       if func is not None:
	  func(id)

       if REQUEST is not None:
	  if RESPONSE is not None:
	    RESPONSE.redirect(REQUEST['HTTP_REFERER'])
	  else:
            return "Cleared"

    security.declareProtected(ManagePayments, 'retractPayment')
    def retractPayment(self, id, REQUEST=None, RESPONSE=None):
       """
	 Retracts a payment for further handling down the line.
	 A payment can have 3 states waiting, retracted and cleared.
	 Proxy method who's implementation is dependant of the Payment
	 Handling.
       """
       funcName = self._getProp('retractPayment')
       func = getattr(self, funcName,None)
       zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
       if func is not None:
	  func(id)
       else:
	  raise "Not declared"

       if REQUEST is not None:
	  if RESPONSE is not None:
	    RESPONSE.redirect(REQUEST['HTTP_REFERER'])
	  else:
            return "Retracted"

#    security.declarePublic('payment')
#    def payment(self, resource, amount, rate):
#	cur = 0
#	funcName = self._getProp('findResource')
#	func = getattr(self, funcName,None)
#	zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
#	if func is not None:
#	  res = func(resource)
#	else:
#	  res = self.UnrestrictedTraverse(resource)
#
#	zLOG.LOG("PaymentTool", 0, "Resource %s" % res)
#	if res is not None:
#	  if self._supliers.has_key(res.Creator()):
#            cur = self._supliers[res.Creator()]
#	  self._supliers[res.Creator()] = cur + amount
	
    security.declarePublic('getDept')
    def getDept(self, resource='', group=''):
	"""
	  Returns the sites dept to the current user.
	"""
	user = _getAuthenticatedUser(self).getId()
	return self.getUserDept(user, resource, group)


    security.declareProtected(ManagePayments, 'getUserDept')
    def getUserDept(self, user='',resource='', group=''):
        """
	  Returns the sites dept to a given user.
	  Proxy method who's implementation is dependant of the Payment
	  Handling.
        """
        funcName = self._getProp('getUserDept')
        func = getattr(self, funcName,None)
        zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
        if func is not None:
	  return func(user=user, resource=resource, group=group)
        else:
	  raise "Not declared"
        
	

    security.declareProtected(ManagePayments, 'payDept')
    def payDept(self, user, REQUEST=None, RESPONSE=None):
       """
         Registers to the system that the dept to the given user
	 has been payed.
	 Proxy method who's implementation is dependant of the Payment
	 Handling.
       """
       funcName = self._getProp('payDept')
       func = getattr(self, funcName,None)
       zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
       if func is not None:
         func(user=user)
       else:
	 raise "Not declared"

       if REQUEST is not None:
	  if RESPONSE is not None:
	    RESPONSE.redirect(REQUEST['HTTP_REFERER'])
	  else:
            return "Retracted"

    #
    # Transaction clearing and registering
    #
    security.declareProtected(ManageTransactions, 'clearTransaction')
    def clearTransaction(self, user, expected=0):
	"""
	  Insures the a user has more credits than is required to authorize
	  access to a given resource.
	"""
	if self.getUserCredits(user) > expected:
	    return true
        else:
	    return false
    
    security.declareProtected(ManageTransactions, 'registerTransaction')
    def registerTransaction(self, user, resource, amount):
	"""
	  Registers a transaction in the system.
	  A transaction is when a user has spend some of his credits on
	  a given resource.
	  Proxy method who's implementation is dependant of the Payment
	  Handling.
	"""
	funcName = self._getProp('registerTransaction')
	func = getattr(self, funcName,None)
	zLOG.LOG("PaymentTool", 0, "Register %s" % funcName)
	if func is not None:
	  func(user=user, resource=resource, amount=amount)

	
Globals.InitializeClass(PaymentTool)

